@@ -6,7 +6,8 @@ The pyoperator package contains the following modules or packages:
 - nonlinear : defines non-linear operators (such as thresholding or rounding)
 - iterative : defines iterative algorithms working with operators
 
-- pywt_operators : (optional) loaded if PyWavelets is present.
+- operators_mpi : (optional) loaded if mpi4py is present.
+- operators_pywt : (optional) loaded if PyWavelets is present.
 """
 
 from .utils import *
@@ -15,10 +16,19 @@ from .linear import *
 from .nonlinear import *
 from .iterative import *
 
-# try to import pywt
 try:
-    from .pywt_operators import *
+    from .operators_pywt import *
 except(ImportError):
     pass
 
-__version__ = '0.2'
+try:
+    from .operators_mpi import *
+except(ImportError):
+    pass
+
+__version__ = '0.3'
+
+import types
+__all__ = [ f for f in dir() if f[0] != '_' and not isinstance(eval(f),
+            types.ModuleType)]
+del types
