@@ -5,12 +5,14 @@ import numpy as np
 from scipy.sparse.linalg import eigsh
 
 from .decorators import linear, real, idempotent, symmetric, inplace
-from .core import (Operator, BroadcastingOperator, DiagonalOperator,
-                   IdentityOperator, ZeroOperator, asoperator)
-from .utils import isscalar
+from .core import (Operator, CompositionOperator, DiagonalOperator,
+                   IdentityOperator, ZeroOperator, DirectOperatorFactory,
+                   ReverseOperatorFactory, asoperator)
+from .utils import isscalar, strshape
 
 __all__ = [
     'BandOperator',
+    'DenseOperator',
     'EigendecompositionOperator',
     'MaskOperator',
     'PackOperator',
@@ -20,6 +22,56 @@ __all__ = [
 ]
 
 
+@linear
+class DenseOperator(Operator):
+    """
+    Operator representing a dense matrix.
+
+    Example
+    -------
+    >>> m = array([[1.,2.,3.],[2.,3.,4.]])
+    >>> d([1,0,0])
+    array([ 1.,  2.])
+    """
+    def __init__(self, data, shapein=None, shapeout=None, dtype=None,
+                 **keywords):
+        if data is None:
+            raise ValueError('The input data is None.')
+        data = np.asarray(data)
+        if dtype is None:
+            dtype = data.dtype
+        data = np.array(data, dtype, copy=False)
+        if data.ndim != 2:
+            raise ValueError('The input is not a 2-dimensional array.')
+        if shapein is None:
+            shapein = data.shape[1]
+        elif np.product(shapein) != data.shape[1]:
+            raise ValueError("The input shape '{0}' is incompatible with that o"
+                             "f the input matrix '{1}'.".format(strshape(
+                             shapein), data.shape[1]))
+        if shapeout is None:
+            shapeout = data.shape[0]
+        elif np.product(shapeout) != data.shape[0]:
+            raise ValueError("The input shape '{0}' is incompatible with that o"
+                             "f the input matrix '{1}'.".format(strshape(
+                             shapeout), data.shape[0]))
+        Operator.__init__(self, shapein=shapein, shapeout=shapeout, dtype=dtype,
+                          **keywords)
+        self.data = data
+        self.set_rule('.C', lambda s: DirectOperatorFactory(type(s), s,
+            np.conjugate(s.data)))
+        self.set_rule('.T', lambda s: ReverseOperatorFactory(type(s), s,
+            s.data.T))
+        self.set_rule('.H', lambda s: ReverseOperatorFactory(type(s), s,
+            np.conjugate(s.data.T)))
+
+    def direct(self, input, output):
+        np.dot(self.data, input.ravel(), output)
+
+    def todense(self, shapein=None):
+        return self.data
+
+
 @real
 @idempotent
 @inplace
@@ -48,17 +100,12 @@ class MaskOperator(DiagonalOperator):
             return ZeroOperator(shapein=shapein, dtype=dtype, **keywords)
         elif np.all(data == 0):
             return IdentityOperator(shapein=shapein, dtype=dtype, **keywords)
-        return BroadcastingOperator.__new__(cls, data, broadcast=broadcast,
-            shapein=shapein, dtype=dtype, **keywords)
+        return DiagonalOperator.__new__(cls)
 
     def __init__(self, mask, dtype=None, **keywords):
         DiagonalOperator.__init__(self, mask, dtype=np.bool8, **keywords)
         self.data = ~self.data
 
-    conjugate_ = None
-    inverse = None
-    inverse_conjugate = None
-
 
 @linear
 @real
@@ -72,15 +119,13 @@ class PackOperator(Operator):
         self.mask = ~np.array(mask, np.bool8, copy=False)
         Operator.__init__(self, shapein=self.mask.shape,
                           shapeout=np.sum(self.mask), **keywords)
-        self.add_rule('.T.', '1')
+        #XXX .T does not share the same mask...
+        self.set_rule('.T', lambda s: UnpackOperator(~s.mask, dtype=s.dtype))
+        self.set_rule('.T.', '1', CompositionOperator)
 
     def direct(self, input, output):
         output[...] = input[self.mask]
 
-    def associated_operators(self):
-        #XXX .T does not share the same mask...
-        return { 'T' : UnpackOperator(~self.mask, dtype=self.dtype) }
-
 
 @linear
 @real
@@ -93,16 +138,13 @@ class UnpackOperator(Operator):
         self.mask = ~np.array(mask, np.bool8, copy=False)
         Operator.__init__(self, shapein=np.sum(self.mask),
                           shapeout=self.mask.shape, **keywords)
-        self.add_rule('.T.', '1')
+        self.set_rule('.T', lambda s: PackOperator(~s.mask, dtype=s.dtype))
+        self.set_rule('.T.', '1', CompositionOperator)
 
     def direct(self, input, output):
         output[...] = 0
         output[self.mask] = input
 
-    def associated_operators(self):
-        #XXX .T does not share the same mask...
-        return {'T' : PackOperator(~self.mask, dtype=self.dtype) }
-
 @linear
 class TridiagonalOperator(Operator):
     def __init__(self, diag, subdiag, superdiag=None, **kwargs):
@@ -161,7 +203,12 @@ class TridiagonalOperator(Operator):
                                              self.superdiag.dtype])
         self.kwargs = kwargs
         flags = {"symmetric":np.all(self.subdiag == self.superdiag)}
-        Operator.__init__(self, shapein=shapein, dtype=self.dtype, flags=flags, **kwargs)
+        Operator.__init__(self, shapein=shapein, dtype=self.dtype, flags=flags,
+                          **kwargs)
+        self.set_rule('.T', lambda s: TridiagonalOperator(self.diag,
+                      self.superdiag, self.subdiag))
+        self.set_rule('.C', lambda s: TridiagonalOperator(np.conj(self.diag),
+                      np.conj(self.subdiag), np.conj(self.superdiag)))
 
     def direct(self, input, output):
         output[:] = self.diag * input
@@ -173,15 +220,6 @@ class TridiagonalOperator(Operator):
         output[:-1] += self.subdiag * input[1:]
         output[1:] += self.superdiag * input[:-1]
 
-    def associated_operators(self):
-        return {'T' : TridiagonalOperator(self.diag,
-                                          self.superdiag,
-                                          self.subdiag),
-                'C' : TridiagonalOperator(np.conj(self.diag),
-                                          np.conj(self.subdiag),
-                                          np.conj(self.superdiag))
-                }
-
     def __repr__(self):
         r = [repr(self.diag), repr(self.subdiag)]
         if self.subdiag is not self.superdiag:
@@ -515,6 +553,7 @@ class EigendecompositionOperator(Operator):
         self.kwargs = kwargs
         Operator.__init__(self, shapein=self.M.shapein, direct=self.M.direct,
                           **kwargs)
+        self.set_rule('.I', lambda s: s ** -1)
 
     def det(self):
         """
@@ -561,6 +600,3 @@ class EigendecompositionOperator(Operator):
         """
         nze = self.eigenvalues[self.eigenvalues != 0]
         return nze.max() / nze.min()
-
-    def associated_operators(self):
-        return { 'I' : self ** -1 }
