@@ -3,10 +3,11 @@ from __future__ import division
 import collections
 import multiprocessing
 import numpy as np
+import operator
 import os
 import scipy.sparse
 
-__all__ = [ 'assignment_operation' ]
+__all__ = [ 'operation_assignment' ]
 
 class ndarraywrap(np.ndarray):
     pass
@@ -61,12 +62,19 @@ def assert_is_not_none(a, msg=None):
         return
     assert False, str(a) + ' is None' + _get_msg(msg)
 
-def assignment_operation(a, b):
+def operation_assignment(a, b):
     """
-    assignment_operation(a, b) -- Same as a[...] = b.
+    operation_assignment(a, b) -- Same as a[...] = b.
     """
     a[...] = b
 
+operation_symbol = {
+    operator.iadd: '+',
+    operator.isub: '-',
+    operator.imul: '*',
+    operator.idiv: '/',
+}
+
 def first_is_not(l, v):
     """
     Return first item in list which is not the specified value.
@@ -172,7 +180,7 @@ def tointtuple(data):
         return data
     try:
         return tuple(None if d is None else int(d) for d in data)
-    except:
+    except TypeError:
         return (int(data),)
 
 def _get_msg(msg):
